/*
 * spi.c
 *
 *  Created on: Nov 6, 2022
 *      Author: hussamaldean
 */


#include <mcp4131_spi.h>
#include "stm32f4xx.h"

void mcp4131_spi_pins_init()
{
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN; //enable clock for GPIOA
	GPIOA->MODER|=GPIO_MODER_MODE0_0;
	GPIOA->MODER&=~GPIO_MODER_MODE0_1;

	GPIOA->MODER|=GPIO_MODER_MODE1_0;
	GPIOA->MODER&=~GPIO_MODER_MODE1_1;

	GPIOA->MODER |=(GPIO_MODER_MODE5_1|GPIO_MODER_MODE6_1|GPIO_MODER_MODE7_1);
	GPIOA->MODER &=~(GPIO_MODER_MODE5_0|GPIO_MODER_MODE6_0|GPIO_MODER_MODE7_0);

	GPIOA->AFR[0]|=(0x05<<20)|(0x05<<24)|(0x05<<28);
}

void mcp4131_spi_config()
{
	/*Enable clock access to SPI1 module*/
		RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;

		/*Set clock to fPCLK/256*/
		SPI1->CR1 |=SPI_CR1_BR_0;
		SPI1->CR1 |=SPI_CR1_BR_1;
		SPI1->CR1 |=SPI_CR1_BR_2;


		/*Enable full duplex*/
		SPI1->CR1 &=~SPI_CR1_RXONLY;

		/*Set MSB first*/
		SPI1->CR1 &= SPI_CR1_LSBFIRST;

		/*Set mode to MASTER*/
		SPI1->CR1 |= SPI_CR1_MSTR;

		/*Set 8 bit data mode*/
		SPI1->CR1 &= ~SPI_CR1_DFF;

		/*Select software slave management by
		 * setting SSM=1 and SSI=1*/
		SPI1->CR1 |= SPI_CR1_SSM;
		SPI1->CR1 |= SPI_CR1_SSI;

		/*Enable SPI module*/
		SPI1->CR1 |= SPI_CR1_SPE;
}

void mcp4131_spi_transmit(uint8_t *data,uint32_t size)
{
	uint32_t i=0;

	while(i<size)
	{
		/*Wait until TXE is set*/
		while(!(SPI1->SR & (SPI_SR_TXE))){}

		/*Write the data to the data register*/
		SPI1->DR = data[i];
		i++;
	}
	/*Wait until TXE is set*/
	while(!(SPI1->SR & (SPI_SR_TXE))){}

	/*Wait for BUSY flag to reset*/
	while((SPI1->SR & (SPI_SR_BSY))){}

	/*Clear OVR flag*/
	(void)SPI1->DR;
	(void)SPI1->SR;
}


void mcp4131_enable(void)
{
	GPIOA->BSRR =GPIO_BSRR_BR9;

}

/*Pull high to disable*/
void mcp4131_disable(void)
{
	GPIOA->BSRR =GPIO_BSRR_BS9;
}
